function [y, W] = ICA(x, alpha = 0.05, maxitr = 100, eta=100.0)
% y = ICA(x, alpha, maxitr, eta)
%
% Applies ICA to mixed signals. 
% Since this algorithm is based on [1], we can use this for complex
% data (but not confirmed).
%
% Inputs
% ------
%    x: channe x length
%    alpha: learning rate
%    maxitr: maximum number of iteration
%
% Outputs
% ------
%    y: separated data
%    W: separation matrix
% 
%
% [1] H. Sawada, R. Mukai, and S. Araki, "Polar Coordinate based Nonlinear
% Function for Frequency-Domain Blind Source Separation,"
% IEICE Trans. Fundamentals, Vol. E86-A, No. 3, 505-510, 2003.
% 
[n_ch n_len] = size(x);

W = eye(n_ch, n_ch);
for i = 1: maxitr
  y = W * x;
  %-- original
  %phi = tanh(eta*abs(y)) .* y ./ abs(y); 
  %--

  %-- simple form
  phi = y ./ abs(y);
  %--

  hcor = eye(n_ch, n_ch) - phi * y' ./ n_len;
  for j = 1: n_ch
    hcor(j,j) = 0;
  end
  W = W + alpha * hcor * W;
end
